import java.lang.Math;

/**
 * Classe de calcul des courbes de vitesse nulle du probleme a 3 corps restreint
 *
 *  @author Benoit Carry & Nicolas Delsate
 *  @version 17/01/2006
 */
public class CourbeVitesseNulle{

  private static double mu;

  /** 
   * Constructeur de la classe
   *
   *  @param param Parametre d'entree de la classe, a savoir la masse reduite du systeme
   *
   */
  public CourbeVitesseNulle(double param){
    mu=param;
  }
  

  /**
   * Procedure de calcul des courbes de vitesse nulle.
   * Le calcul s'effectue en quadrillant le plan et en calculant la vitesse en tout point. 
   *
   *  @param jacobInit Valeur de l'integrale de Jacobi initiale du probleme, suivant les conditions initiales
   *  @param precisionZero Maille du quadrillage utilise pour calculer les courbes de vitesse nulle
   */
  public static double[] calculCourbeV0(double jacobInit,double precisionZero) {

    
    int nbrPas= 1800;
    double precisionGrillage;   //=1.0e-2;
    double taille= 2.0;      //-Demi-taille de la zone couverte
    double q1min = -taille;  //-Limite inferieure en q1
    double q1max = taille;   //-Limite superieure en q1
    double q2min = 0.0;      //-Limite inferieure en q2
    double q2max = taille;   //-Limite superieure en q2

   //-Maillage du grillage en fonction du nombre de pas demandes
    precisionGrillage=(q1max-q1min) / nbrPas;

   //-Conditions initiales
    double q1=q1min;
    double q2=q2min;


   //-Preparation des sorties
    double[] v0q1=new double[1000000];
    double[] v0q2=new double[1000000];
    v0q1[0]=0.0;


    int compteur=0;

    double vCarre;
    double R1;
    double R2;

   //-Boucle sur le maillage, premiere coordonnee
    for(int i=0; i<nbrPas; i++){

     //-Evolution des positions
      q1=q1+precisionGrillage;
      q2=q2min;

     //-Boucle sur le maillage, seconde coordonnee
      for(int j=0; j<nbrPas/2; j++){

       //-Evolution des positions
        q2=q2+precisionGrillage;

       //-Calcul des distances aux masses
        R1=Math.sqrt( Math.pow(q1+mu,2.0) + Math.pow(q2,2.0) );
        R2=Math.sqrt( Math.pow(q1+mu-1.0,2.0) + Math.pow(q2,2.0) );

       //-Calcul du carre de la vitesse
        vCarre=Math.pow(q1,2.0)+Math.pow(q2,2.0)+2*( (1.0-mu)/R1 + mu/R2 ) - jacobInit;

       //-Repere les courbes de vitesse nulle
        if( (vCarre <= 0.0)  && (vCarre > -precisionZero) ){

          compteur++;

         //-Enregistrement de la position du point appartenant a la courbe
          v0q1[compteur]=q1;
          v0q2[compteur-1]=q2;

        }
      }
    }


   //-Premiere valeur du tableau vaut le nombre de points necessaires pour tracer la courbe
    double[] resultat=new double[4*compteur + 1];
    resultat[0]=2*compteur;

   //-Reorganisation des points de maniere a obtenir egalement les points en q2 negatif
    for( int i=1; i<compteur ; i++ ){

      resultat[i]=v0q1[i];
      resultat[compteur+i]=v0q1[i];
      resultat[2*compteur+i]=v0q2[i-1];
      resultat[3*compteur+i]=-v0q2[i-1];

    }

    return resultat;

  }

}
