import java.lang.Math;
/**
 * classe qui implemente l'equation d'un mouvement et une intergale premiere
 *
 * Implemente la classe InterfaceEqMvmt
 *
 *@author Nicolas DELSATE
 *@version
 * Derniere modification :
 *  08/12/2005
 */
public class EqMvmtPblmTroisCorps implements InterfaceEqMvmt{

  //////////////////////////////////////////////////////////////////
  //                          eqMvmt                              //
  //////////////////////////////////////////////////////////////////

  /**
   * fonction utilise dans rungeKutta pour valuer la valeur en un point
   * @param  x            un tableau de la forme [Q1,Q2,P1,P2]
   * @param  parametres   tableau de parametres
   * @return   un tableau de la forme [Q1_dot,Q2_dot,P1_dot,P2_dot] 
   */

  public double[] eqMvmt(double[] x, double[] parametres){
    /*x[0]=Q1
      x[1]=Q2
      x[2]=Q1dot
      x[3]=P2
    */

    //tableau (de longueur de x :x.length ) contenant la solution
    double[] xdot=new double[x.length];

    //le tableau de parametre n'est qu'un nombre appele mu
    double mu=parametres[0];

    //calcul de Qdot1=xdot[0]
    xdot[0]=x[2]+x[1];
    //---Debugging----
    //System.out.println("------------------------");
    //System.out.println("--x[0] = " + x[0] + " et  xdot[0] = " + xdot[0]);
    //---Debugging-

    //calcul de Qdot2=xdot[1]
    xdot[1]=x[3]-x[0];
    //---Debugging----
    //System.out.println("--x[1] = " + x[1] + " et  xdot[1] = " + xdot[1]);
    //---Debugging-

    //derivee de R1 par rapport a Q1
    double denom1 = Math.pow( Math.pow(x[0]+mu,2.0)+Math.pow(x[1],2.0) , 1.5 );

    //derivee de R2 par rapport a Q2
    double denom2 = Math.pow( Math.pow(x[0]+mu-1,2.0)+Math.pow(x[1],2.0) , 1.5 );

    //calcul de Pdot1=xdot[2]
    xdot[2] = x[3] - (1.0-mu)*(x[0]+mu) / denom1  - mu*(x[0]+mu-1) / denom2 ;
    //---Debugging----
    //System.out.println("--x[2] = " + x[2] + " et  xdot[2] = " + xdot[2]);
    //---Debugging-
 
    //calcul de Pdot1=xdot[3]
    xdot[3] = -x[2] - (1-mu)*x[1]/denom1  -  mu*x[1]/denom2 ;

    //---Debugging----
    //System.out.println("--x[3] = " + x[3] + " et  xdot[3] = " + xdot[3]);
    //---Debugging-

    return xdot;
  }







  //////////////////////////////////////////////////////////////////
  //                          intPrem                             //
  //////////////////////////////////////////////////////////////////

   /**
   * Implemente une integrale premiere
   * @param  x            un tableau de la forme [X1,X2,...]
   * @param  parametres   tableau de parametres
   * @return              la valeur de l'integrale premiere
   */
  public double intPrem(double[] x, double[] parametres)
    {
      double R1=Math.pow(Math.pow(x[0]+parametres[0],2)+Math.pow(x[1],2),0.5);
      double R2=Math.pow(Math.pow(x[0]+parametres[0]-1,2)+Math.pow(x[1],2),0.5);
      double lagrangien = 0.5* (  Math.pow(x[2]+x[1],2) + Math.pow(x[3]-x[0],2) ) - 0.5 * (Math.pow(x[0],2) + Math.pow(x[1],2) ) - ( (1-parametres[0])/R1 +  parametres[0]/R2 );

      double jacobi=-2.0*lagrangien;

      return jacobi;
    }

}
